/*
 * Race track generator
 *
 * leto/calodox, 02.2002
 */

#include <opengl.h>

#include "fxtrackgen.h"
#include "trackgen.h"

FXTrackGen *trackGenPtr=0;

FXTrackGen::FXTrackGen() :
	zoom(1)
{
	pos[0]=0;
	pos[1]=0;
	pos[2]=-1;

	for (unsigned int i=0; i<4; i++)
		for(unsigned int j=0; j<4; j++)
			if (i==j) rot[i][j]=1; 
			else rot[i][j]=0;

	tg.nbSeg = 10;
	tg.curveTrig = 0.97;
	tg.keyDist = 10;
	level = 1;
	enable=true;
}

void FXTrackGen::registerOptions() {
	register3DPosOpt("Position",0,pos);
	register3DRotOpt("Rotation",0,rot);
	registerFloatOpt("Zoom", 0, &zoom, 0.001f, 20.0f);
	registerIntOpt("# segments", 0, &tg.nbSeg, 1,100);
	registerFloatOpt("Curving trigger", 0, &tg.curveTrig, 0, 1);
	registerBoolOpt("Enable Rendering", 0, &enable);

	registerIntOpt("Complexity level", OPTION_REINIT, &level, 0, 20);
	registerFloatOpt("Key distance",OPTION_REINIT, &tg.keyDist);
}

int FXTrackGen::init() {
	tg.generateKeys(level);
	return 1;	
}

int FXTrackGen::render() {

	if (enable) {
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustum(-1,1,-1,1,1,10);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(pos[0], pos[1], pos[2]);
	glMultMatrixf(&rot[0][0]);
	glScalef(zoom, zoom, zoom);

	glDisable(GL_BLEND);
	glBegin(GL_TRIANGLES);

	trackGenPtr = this;
	tg.generateTrack();

	glEnd();
	}
	return 1;
}

void FXTrackGen::finish() {
}

void addTriangle(float *v1, float *v2, float *v3, int type) {

	switch (type) {
	case 0: // normal road
		glColor4f(.7f, .7f, .7f, 1);
		break;
	case 1: // normal border
		glColor4f(0.8f, 1.0f, 0.5f, 1);
		break;
	case 2: // turning border
		glColor4f(1.0f, 1.0f, 0, 1);
		break;
	default:
		glColor4f(1, 0, (float)type/255.0f, 1);
	}

	glVertex3fv(v1);
	glVertex3fv(v2);
	glVertex3fv(v3);
}
