/*
 * Calodox Demoniac duration abstraction support.
 *
 * Julien Pilet <leto@calodox.org>, 12.2001
 */

#include "duration.h"
#include "durationcontainer.h"
#include <string.h>

Duration::~Duration() {
	if (name)
		delete[] name;
}

void Duration::setName(const char *name) {
	if (this->name)
		delete[] this->name;

	if (name && name[0]) {
		this->name = new char[strlen(name) +1];
		strcpy(this->name, name);
	} else {
		this->name = 0;
	}
}

void Duration::setInitTime(int time) {

	initTime = time; 
	startTime = time > startTime ? time : startTime;
}

void Duration::setStartTime(int time) { 
	startTime = time; 
	initTime = startTime < initTime ? startTime : initTime; 
}

void Duration::setDuration(int time) {
	if (time >= 0)
		duration = time;
	else 
		duration = 0;
}

int Duration::absToRel(int time) {
	int t = time;

	for (Duration *d = this; d->father !=0; d = d->father) {
		t -= d->startTime;
	}
	return t;
}

int Duration::relToAbs(int time) {
	int t = time;

	for (Duration *d = this; d->father !=0; d = d->father) {
		t += d->startTime;
	}
	return t;
}

void Duration::load(Loader *l) {
	l->checkType(Loader::OBJECT);
	l->checkName("Duration");
	l->nextLine();

	char n[512];
	l->getString(n, sizeof(n), "name");
	setName(n);
	l->nextLine();

	initTime = l->getInt("initTime");
	l->nextLine();
	
	startTime = l->getInt("startTime");
	l->nextLine();
	
	duration = l->getInt("duration");
	l->nextLine();

	l->checkType(Loader::OBJECT_END);
	l->nextLine();
}

void Duration::save(Saver *s) {
	s->saveObjectBegin("Duration");

	s->saveString("name", name);
	s->saveInt("initTime", initTime);
	s->saveInt("startTime", startTime);
	s->saveInt("duration", duration);
	s->saveObjectEnd();
}
