/*
 * Calodox Demoniac duration abstraction class.
 *
 * Julien Pilet <leto@calodox.org>, 12.2001
 */

#ifndef _DURATION_H_
#define _DURATION_H_

#include "saver.h"
#include "loader.h"

class DurationContainer;

//! the Duration class represents something that starts & stop at given 
// times. It can be an effect instance, or a container for other durations.
class Duration {
protected:
	bool initialized;

public:
	
	//! the name of this duration. Might be 0.
	char *name;

	int initTime;
	int startTime;
	int duration;

	//! GUI available ptr. GtkTreeItem for Gtk.
	void *guiData;

	DurationContainer *father;

	Duration() : initialized(false), name(0), 
		initTime(0), startTime(0), duration(10000), guiData(0),
		father(0) {}

	Duration(int i, int s, int d) : initialized(false), name(0),
		initTime(i), startTime(s), duration(d), guiData(0),
		father(0) {}

	virtual ~Duration();

	//! returns true if the class is a container
	virtual bool isContainer() { return false; }

	//! change the name of this duration
	virtual void setName(const char *name);

	//! prepare everything to be ready to do actionAtTime()
	//  for effect instances, compute options.
	virtual void setTime(int time)=0;

	//! do what should be done at the given time
	virtual void actionAtTime(int time) = 0;

	//! initialize the duration.
	virtual void init() { initialized = true; }

	//! change the init time. If the new time is greater than
	// the start time, start time will be changed to initTime.
	virtual void setInitTime(int time);

	//! sets the starting time making sure 
	// initTime <= startTime  
	virtual void setStartTime(int time);

	//! Set the duration. time must be positive or null.
	virtual void setDuration(int time);

	//! Convert an absolute time to a time relative
	// to the startTime of this duration.
	int absToRel(int time);

	//! Convert a time relative to the startTime of
	// this duration to an absulute time.
	int relToAbs(int time);

	// Save & load operations
	virtual void save(Saver *s);
	virtual void load(Loader *l);
};


#endif

