/*
 * Duration Container for calodox Demoniac
 *
 * Julien Pilet <leto@calodox.org>, 12.2001
 */

#include <vector>

#include "durationcontainer.h"
#include "report.h"
#include "fxinstance.h"
#include "glmain.h"

DurationContainer::DurationContainer(bool loop)
: isLoop(loop)
{ 
	if (loop)
		setName("Effect loop"); 
	else 
		setName("Effect group");
}

Duration *DurationContainer::constructor() {
	return new DurationContainer(false);
}

Duration *DurationContainer::loopConstructor() {
	return new DurationContainer(true);
}

DurationContainer::~DurationContainer() {
	std::vector<Duration *>::iterator it;	
	
	for (it = children.begin(); it != children.end(); ++it) 
		delete *it;	
}

void DurationContainer::addChild(Duration *child) {
	children.push_back(child);
	child->father = this;
}

void DurationContainer::removeChild(Duration *child) {
	std::vector<Duration *>::iterator it;	
	
	for (it = children.begin(); it != children.end(); ++it) {
		if (*it == child) {
			delete *it;
			children.erase(it);
			break;
		}
	}
}

void DurationContainer::clearAll() {
	while (children.size()) 
		removeChild(children[0]);
}

void DurationContainer::setTime(int time) {
	int relTime = time - startTime;
	
	if (relTime >= 0 && relTime <= duration) {
		std::vector<Duration *>::iterator it;	

		for (it = children.begin(); it != children.end(); ++it) 
			(*it)->setTime(getLoopTime(*it, relTime));
	}
}

void DurationContainer::actionAtTime(int time) {

	if (!initialized && time >= initTime) {
		init();
	} 

	if (time >= startTime && time <= startTime + duration) {
		int rel_time = time - startTime;

		std::vector<Duration *>::iterator it;	

		for (it = children.begin(); it != children.end(); ++it) {
			(*it)->actionAtTime(getLoopTime(*it, rel_time));
		}
	}
}

void DurationContainer::init() {
	if (initialized)
		return;

	std::vector<Duration *>::iterator it;	
	
	for (it = children.begin(); it != children.end(); ++it) 
		(*it)->init();

	initialized = true;
}

void DurationContainer::load(Loader *l) {

	l->checkType(Loader::OBJECT);
	l->checkName("DurationContainer");
	l->nextLine();

	Duration::load(l);

	isLoop = (bool) l->getInt("isLoop");
	l->nextLine();

	if (father != 0)
		glMainptr->trackDlg.addDuration(this);

	while (l->nextType() == Loader::OBJECT) {
		Duration *d;

		if (strcmp(l->nextName(), "FXInstance") == 0) {
			d = new FXInstance(l, glMainptr->vfx);
			addChild(d);
			glMainptr->trackDlg.addDuration(d);
		} else {
			d = glMainptr->durationFactory.create(l->nextName());
			if (d==0) l->error("Can't load duration !");
			addChild(d);
			d->load(l);
			glMainptr->trackDlg.addDuration(d);
		}	
	}
	
	l->checkType(Loader::OBJECT_END);
	l->nextLine();
}

void DurationContainer::save(Saver *s) {

	s->saveObjectBegin("DurationContainer");

	Duration::save(s);

	s->saveInt("isLoop", (int) isLoop);

	std::vector<Duration *>::iterator it;	
	
	for (it = children.begin(); it != children.end(); ++it) 
		(*it)->save(s);

	s->saveObjectEnd();
}

int DurationContainer::getLoopTime(Duration *d, int time) {
	if (isLoop) {
		int t = time - d->startTime;
		t= t % d->duration;
		return d->startTime + t;
	} else {
		return time;
	}
}
