/*
 * Duration Container for calodox Demoniac
 *
 * Julien Pilet <leto@calodox.org>, 12.2001
 */

#ifndef _DURATIONCONTAINER_H_
#define _DURATIONCONTAINER_H_

#include <vector>

#include "duration.h"
#include "loader.h"
#include "saver.h"

//! A container for effects. 

// It can be just a single container
// or a looping container.
class DurationContainer : public Duration {
private:
	//! computes the time
	int getLoopTime(Duration *d, int time);

	//! true if children should loop
	bool isLoop;

public:

	//! the children, if any.
	std::vector<Duration *> children;

	//! the constructor. If loop is true,
	//  the container will make effects loop.
	DurationContainer(bool loop=false);

	//! just a 'new DurationContainer(false);'
	static Duration *constructor();

	//! just a 'new DurationContainer(true);'
	static Duration *loopConstructor();

	//! return true if this container loops,
	// false if not.
	bool getLoop() { return isLoop; }

	//! set or unset the looping feature.
	void setLoop(bool loop) { isLoop = loop; }

	virtual ~DurationContainer();

	virtual bool isContainer() { return true; }
	virtual void addChild(Duration *child);
	virtual void removeChild(Duration *child);
	virtual void clearAll();
	
	//! propagate init() through all children and mark as initialized.
	virtual void init();

	//! propagate setTime through all children
	virtual void setTime(int time);

	//! propagate the event trough all children.
	// the action is propagated only if this duration is active,
	// or if the time is -1. Time propagation is relative to 
	// startTime.
	virtual void actionAtTime(int time);

	void load(Loader *l);
	void save(Saver *s);
};

#endif

