
#include <assert.h>
#include "durationdlg.h"
#include "glmain.h"

DurationDlg::DurationDlg() : duration(0)
{
}

DurationDlg::~DurationDlg() {
}

void DurationDlg::build() {
	
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			GTK_SIGNAL_FUNC(Dlg::cbDeleteWindow), this);

	gtk_container_set_border_width(GTK_CONTAINER(window), 5);

	GtkWidget *vbox = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(window), vbox);

	// The name entry
	GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
	
	GtkWidget *label = gtk_label_new("Name:");
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

	nameEntry = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), nameEntry, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(nameEntry), "changed",
			GTK_SIGNAL_FUNC(cbRenameDuration), this);

	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, FALSE, 0);

	// the init time entry
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Initialization time:"),
			FALSE, FALSE, 0);
	initTimeAdj = GTK_ADJUSTMENT(gtk_adjustment_new(0,0,FLT_MAX, 1, 10, 0));
	GtkWidget *spin = gtk_spin_button_new(initTimeAdj, 1, 3);
	gtk_signal_connect(GTK_OBJECT(initTimeAdj), "value_changed",
			GTK_SIGNAL_FUNC(cbInitChanged), this);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, FALSE, 0);

	// the start time entry
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Start time:"),
			FALSE, FALSE, 0);
	startTimeAdj = GTK_ADJUSTMENT(gtk_adjustment_new(0,0,FLT_MAX, 1, 10, 0));
	spin = gtk_spin_button_new(startTimeAdj, 1, 3);
	gtk_signal_connect(GTK_OBJECT(startTimeAdj), "value_changed",
			GTK_SIGNAL_FUNC(cbStartChanged), this);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, FALSE, 0);

	// the duration entry
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Duration:"),
			FALSE, FALSE, 0);
	durationAdj = GTK_ADJUSTMENT(gtk_adjustment_new(0,0,FLT_MAX, 1, 10, 0));
	spin = gtk_spin_button_new(durationAdj, 1, 3);
	gtk_signal_connect(GTK_OBJECT(durationAdj), "value_changed",
			GTK_SIGNAL_FUNC(cbDurationChanged), this);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, FALSE, 0);

	// build the frame for the DurationContainer properties
	containerFrame = gtk_frame_new("Group properties");
	gtk_box_pack_start(GTK_BOX(vbox), containerFrame, TRUE, TRUE, 5);
	
	GtkWidget *fVBox = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(containerFrame), fVBox);
	loopCheckButton = gtk_check_button_new_with_label("loop");
	gtk_box_pack_start(GTK_BOX(fVBox), loopCheckButton, TRUE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(loopCheckButton), "toggled",
		GTK_SIGNAL_FUNC(cbLoopToggled), this);

	
	hbox = gtk_hbox_new(FALSE, 5);

	GtkWidget *ok = gtk_button_new_with_label("Close");

	gtk_signal_connect(GTK_OBJECT(ok), "clicked",
		GTK_SIGNAL_FUNC(Dlg::cbHide), this);

	gtk_box_pack_start( GTK_BOX(hbox), ok, TRUE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	

	gtk_widget_show_all(vbox);
}

const char *DurationDlg::getName() {
	return "Duration properties";
}

void DurationDlg::setDuration(Duration *d) {
	duration = d;

	if (d == 0)
		hide();
	else {
		if (d->isContainer()) 
			gtk_widget_show(GTK_WIDGET(containerFrame));
		else
			gtk_widget_hide(GTK_WIDGET(containerFrame));

		updateGui();
	}
}

void DurationDlg::updateGui() {
	if (duration!=0) {
		gtk_entry_set_text(GTK_ENTRY(nameEntry), duration->name);
		initTimeAdj->value = ((float)duration->initTime)/1000.0;
		startTimeAdj->value = ((float)duration->startTime)/1000.0;
		durationAdj->value = ((float)duration->duration)/1000.0;

		gtk_signal_emit_by_name(GTK_OBJECT(initTimeAdj), "value_changed");
		gtk_signal_emit_by_name(GTK_OBJECT(startTimeAdj), "value_changed");
		gtk_signal_emit_by_name(GTK_OBJECT(durationAdj), "value_changed");
		char t[256];
		sprintf(t, "%s properties", duration->name);
		gtk_window_set_title(GTK_WINDOW(window), t);
		
		if (duration->isContainer()) {
			DurationContainer *dc = (DurationContainer *) duration;
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(loopCheckButton),
				dc->getLoop());
		}
	}
}

void DurationDlg::cbRenameDuration(GtkWidget *widget, DurationDlg *dlg) {

	assert(dlg->duration);

	dlg->duration->setName(gtk_entry_get_text(GTK_ENTRY(widget)));

	char t[256];
	sprintf(t, "%s properties", dlg->duration->name);
	gtk_window_set_title(GTK_WINDOW(dlg->window), t);

	glMainptr->trackDlg.renameDuration(dlg->duration);
}

void DurationDlg::cbInitChanged(GtkAdjustment *widget, DurationDlg *dlg) {
	if (dlg->duration) {
		int intTime = (int)(widget->value*1000.0);
		if (intTime != dlg->duration->initTime) {
			dlg->duration->setInitTime(intTime);
			dlg->updateGui();
		}
	}
}

void DurationDlg::cbStartChanged(GtkAdjustment *widget, DurationDlg *dlg) {
	if (dlg->duration) {
		int intTime = (int)(widget->value*1000.0);
		if (intTime != dlg->duration->startTime) {
			dlg->duration->setStartTime(intTime);
			glMainptr->trackDlg.updateDuration(dlg->duration);
			dlg->updateGui();
		}
	}
}

void DurationDlg::cbDurationChanged(GtkAdjustment *widget, DurationDlg *dlg) {
	if (dlg->duration) {
		int intTime = (int)(widget->value*1000.0);
		if (intTime != dlg->duration->duration) {
			dlg->duration->setDuration(intTime);
			glMainptr->trackDlg.updateDuration(dlg->duration);
			dlg->updateGui();
		}
	}
}

void DurationDlg::cbLoopToggled(GtkToggleButton *toggle, DurationDlg *dlg) {
	assert(dlg !=0 && dlg->duration && dlg->duration->isContainer());

	DurationContainer *dc = (DurationContainer *) dlg->duration;

	dc->setLoop( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle)) );
	glMainptr->reDraw();
}