/*
 * Duration factory for calodox demoniac
 *
 * Julien Pilet <leto@calodox.org> 12.2001
 */

#include "durationfactory.h"
#include "durationcontainer.h"
#include <string.h>

#include "glmain.h"
#include "report.h"

DurationMaker::DurationMaker(DurationConstructor c, const char *name, const char *description, durationType type) 
	: constructor(c), type(type)
{
	this->name = new char[strlen(name) + 1];
	strcpy(this->name, name);

	this->description = new char[strlen(description) + 1];
	strcpy(this->description, description);
}

DurationMaker::~DurationMaker() {
	delete name;
	delete description;
}

DurationFactory::DurationFactory() {
	addDurationMaker( new DurationMaker(DurationContainer::constructor,
				"DurationContainer",
				"Group"));
	addDurationMaker( new DurationMaker(DurationContainer::loopConstructor,
				"DurationLoop",
				"Loop"));
}

DurationFactory::~DurationFactory() {

	std::vector<DurationMaker *>::iterator it;

	for (it = makers.begin(); it != makers.end(); ++it) {
		delete *it;
	}
}

void DurationFactory::addDurationMaker(DurationMaker *maker) {
	makers.push_back(maker);
}

Duration *DurationFactory::create(const char *name, Loader *l) {
	std::vector<DurationMaker *>::iterator it;

	for (it = makers.begin(); it != makers.end(); ++it) {
		if (strcmp((*it)->name, name)==0) {
			return (*it)->constructor();
		}
	}

	// we did'nt find any DurationMaker matching the name.
	// let's look in the effect list.
	std::vector<FXGeneric *>::iterator fx;

	if (l) {
		return new FXInstance(l, glMainptr->vfx);
	} else {
		for (fx = glMainptr->vfx.begin(); fx!= glMainptr->vfx.end(); ++fx) {

			if (strcmp((*fx)->getName(), name)==0) {
				// we found an effect !
				return new FXInstance((*fx)->create());
			}
		}
	}
	
	fprintf(stderr, "%s: ", name);
	fatal("unknown duration");
	return 0;
}


