/*
 * Calodox Demoniac file support
 * (c) 2001 Stephane Magnenat, Julien Pilet, Ysagoon
 */

#include "filemanager.h"
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#else
#define PACKAGE_DATA_DIR ".."
#define PACKAGE_SOURCE_DIR "../.."
#endif

FileManager::FileManager()
{
    addDir(".");
    addDir(PACKAGE_DATA_DIR);
    addDir(PACKAGE_SOURCE_DIR);
}

FileManager::~FileManager()
{
	std::vector<char *>::iterator it;
	for (it=dirList.begin(); it!=dirList.end(); ++it)
	{
		delete[] const_cast<char*>(*it);
	}
}


void FileManager::addDir(const char *dir) 
{
	int len=strlen(dir);
	char *newDir=new char[len+1];
	strcpy(newDir, dir);
	dirList.push_back(newDir);
}

FILE *FileManager::open(const char *filename, const char *subdir, const char *mode)
{

	std::vector<char *>::iterator it;
	for (it=dirList.begin(); it!=dirList.end(); ++it)
	{
		char *fn;

		if (subdir) {
			fn = new char[strlen(filename) + strlen(subdir) + strlen(*it) + 4];
			sprintf(fn, "%s%c%s%c%s", *it, DIR_SEPARATOR ,
					subdir, DIR_SEPARATOR, filename);
		} else {
			fn = new char[strlen(filename) + strlen(*it) + 3];
			sprintf(fn, "%s%c%s", *it, DIR_SEPARATOR ,filename);
		}

		FILE *fp =  fopen(fn, mode);
		delete[] fn;
		if (fp) 
			return fp;
	}

	return NULL;
}

char *FileManager::find(const char *filename, const char *subdir, const char *mode)
{

	std::vector<char *>::iterator it;
	for (it=dirList.begin(); it!=dirList.end(); ++it)
	{
		char *fn;

		if (subdir) {
			fn = new char[strlen(filename) + strlen(subdir) + strlen(*it) + 4];
			sprintf(fn, "%s%c%s%c%s", *it, DIR_SEPARATOR ,
					subdir, DIR_SEPARATOR, filename);
		} else {
			fn = new char[strlen(filename) + strlen(*it) + 3];
			sprintf(fn, "%s%c%s", *it, DIR_SEPARATOR ,filename);
		}

		FILE *fp =  fopen(fn, mode);
		if (fp) { 
			fclose(fp);
			return fn;
		}
		delete[] fn;
	}

	return NULL;
}
