/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * FXInstance: effect instance class for DemoNiac
 * leto/calodox, 18.3.2001
 */

#include <assert.h>
#include <stdio.h>

#include <fx.h>
#include "glmain.h"
#include "dlgopt.h"
#include "fxinstance.h"
#include "report.h"

FXInstance *currentFX; 

FXInstance::FXInstance(FXGeneric *fx) { 

	this->fx = fx;
	dlg = 0;

	fxInitialized = false;

	if (fx == 0) {
		warning("null fx pointer at instance creation !");
		return;
	}

	// register all fx options
	currentFX= this;
	fx->registerOptions();
	currentFX= 0;

	setName(fx->getName());
	
	dlg = new DlgOpt(this);
	dlg->build();
	dlg->show();
}

FXInstance::~FXInstance() {
	finish();

	for (unsigned int i=0; i<opts.size(); i++)
		delete opts[i];

	delete dlg;
	delete fx;
}

// loader constructor
FXInstance::FXInstance(Loader *l, vector <FXGeneric *> vfx) {
	unsigned int i;

	dlg = 0;

	fxInitialized = false;

	l->checkType(Loader::OBJECT);
	l->checkName("FXInstance");
	l->nextLine();

	Duration::load(l);
	
	// look for the fx
	char fxName[128];
	l->getString(fxName, 127, "effectName");
	l->nextLine();

	for (i=0; i<vfx.size(); i++) {
		if (strcmp(fxName, vfx[i]->getName())==0) {
			// create a new instance
			fx = vfx[i]->create();
			init();
			break;
		}
	}

	// load options
	currentFX= this;
	fx->registerOptions();
	currentFX= 0;

	//glMainptr->trackDlg.addFXInstance(this);

	for (i=0; i < opts.size(); i++) {
		l->checkType(Loader::OBJECT);
		opts[i]->load(l);
		//glMainptr->trackDlg.addOption(opts[i]);
	}
	
	assert(l->nextType()==Loader::OBJECT_END);
	l->nextLine();

	// Create dialog
	dlg = new DlgOpt(this);
	dlg->build();
}
    
void FXInstance::save(Saver *s) {
    s->saveObjectBegin("FXInstance");

    Duration::save(s);

    // save the fx
    s->saveString("effectName", fx->getName());

    // save options
    for (unsigned int i=0; i<opts.size(); i++) 
	opts[i]->save(s);
	
    s->saveObjectEnd();
}

void FXInstance::init() {
	if (fxInitialized) 
		fx->finish();
	
	fx->init();
	fxInitialized = true;
}

void FXInstance::finish() {
        if (fxInitialized) {
		fx->finish();
	}
	fxInitialized=false;
}

void FXInstance::render() {
	if (fxInitialized)
		fx->render();
}

void FXInstance::setTime(int time) {

	if (isActive(time)) {
		std::vector<GenericOption *>::iterator it;

		for (it = opts.begin(); it != opts.end(); ++it) {
			(*it)->computeValue(time-startTime);
			(*it)->updateGui();
		}
	}
}

void FXInstance::actionAtTime(int time) {

	if (!fxInitialized && time >= initTime)
		init();

	if (time >= startTime && time <= startTime + duration) {
		render();
	}

	if (time >= startTime + duration)
		finish();
}

bool FXInstance::isActive(int time) {
	return time >= startTime && time < startTime + duration;
}

