/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * GLMain.cpp: GLDemoNiac main class 
 * Leto/calodox, 12.2000
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#else
#define PACKAGE_DATA_DIR "."
#define PACKAGE_SOURCE_DIR "../.."
#endif

#include "opengl.h"
#include <GL/glu.h>
#include <gtk/gtk.h>
#include <gtkgl/gtkglarea.h>
#include <gldemoniac.h>
#include "saver.h"
#include "loader.h"
#include "glmain.h"
#include "fxinstance.h"
#include "timer.h"
#include "menu.h"
#include "drawbufflist.h"
#include "filemanager.h"

#include <assert.h>

// 10 seconds is the minimum demo duration
#define MINIMUM_DEMO_DURATION 10000

// global object pointer for callback functions and other classes.
GLMain *glMainptr=0;

/******************************************
**        GLDemoniac Hide Class          **
*****************************************/

GLDemoniac::GLDemoniac() {
	this->glMainPtr = new GLMain();
}

GLDemoniac::~GLDemoniac() {
	delete this->glMainPtr;
}

int GLDemoniac::registerFX(FXGeneric *fx, int fxsize) {
	return this->glMainPtr->registerFX(fx, fxsize);
}

int GLDemoniac::start(int *argc, char ***argv) {
	return this->glMainPtr->start(argc, argv);
}

void GLDemoniac::reDraw() {
	this->glMainPtr->reDraw();
}
	

/* Constructor: called before any fx registration */
GLMain::GLMain(void) {
	glMainptr = this;
	started = 0;
	selectedFX = 0;
	playing = false;
	demoTimeSec = 0.0;
	filename = 0;
	glMainptr->idleRegistred = false;
}

// finish all effects
GLMain::~GLMain() {
	clearAll();
}

/***************************************
 *      DemoNiac Business Logic        *
 **************************************/

/*
 * Effect instances manipulation
 */

void GLMain::clearAll() {
	
	std::vector<Duration *>::iterator it;
	for (it = demo.children.begin(); it != demo.children.end(); ++it)
		trackDlg.removeDuration(*it);
	demo.clearAll();

	started = 0;

	if (playing) 
		cbPlayStopButton(0, this);
	if (filename) {
		delete[] filename;
		filename = 0;
	}
	setTime(0);

	drawBufferList.clear();
}

int GLMain::registerFX(FXGeneric *fx, int fxsize) {

	vfx.push_back(fx);
	return 0;
}

static bool ignoreSetTime = false;

void GLMain::setTime(float seconds) {

	if (ignoreSetTime || demoTimeSec == seconds)
		return;
	ignoreSetTime = true;

	demoTimeSec = seconds;

	int time = (int) (demoTimeSec * 1000.0);

	// compute options for the new time
	demo.setTime(time);

	// display the new time
	if (timeAdjustment->value != seconds) {
		timeAdjustment->value = seconds;
		gtk_signal_emit_by_name(GTK_OBJECT(timeAdjustment), "value_changed");
	}
	ignoreSetTime = false;
}

void GLMain::setTime(int millisec) {
	setTime(((float)millisec) / 1000.0f );
}	

/******************************************
 *           Callback functions           *
 *****************************************/

/* display call back function. */
gint cbDisplay(GtkWidget *widget, GdkEventExpose *event) {
	glMainptr->reDraw();
	return TRUE;
}

/* init GL callback */
gint cbInit(GtkWidget *widget) {
	if (gtk_gl_area_make_current(GTK_GL_AREA(widget))) { 
		glEnable(GL_DEPTH_TEST);
		glViewport(0,0, widget->allocation.width, widget->allocation.height);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0,100, 100,0, -1,1);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
	}

	glMainptr->reDraw();
	return TRUE;
}

// play / stop button call back
gint cbPlayStopButton(GtkWidget *widget, gpointer data) {

	if (glMainptr->playing) {
		// stop playing
		glMainptr->playing = false;

		glMainptr->toolbar.setPlayStopLabel("Play");

		// cbIdle will be called soon and will
		// - remove itself from idle call list
		// - repaint the screen correctly
	} else {
		// start playing
		glMainptr->playing = true;
		glMainptr->toolbar.setPlayStopLabel("Stop");
		glMainptr->timeStart = getMilliSec() - (int) (glMainptr->demoTimeSec * 1000.0);
		
		// When idling: play the demo.
		glMainptr->idleTag=gtk_idle_add_priority(GTK_PRIORITY_LOW, GLMain::cbIdle, glMainptr);
		glMainptr->idleRegistred = true;
	}
	return TRUE;
}

/* reshape call back function */
gint cbReshape(GtkWidget *widget, GdkEventConfigure *event)
{
	// OpenGL functions can be called only if make_current returns true 
	if (gtk_gl_area_make_current(GTK_GL_AREA(widget))) { 

		int x = widget->allocation.width;
		int y = widget->allocation.height;

		float xy_aspect;

		xy_aspect = (float)x / (float)y;
		glViewport( 0, 0, x, y );

		/* no real 3d
		   glMatrixMode( GL_PROJECTION );
		   glLoadIdentity();
		   glFrustum( -xy_aspect*.08, xy_aspect*.08, -.08, .08, .1, 15.0 );
		 */
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(-0.5,318.5,199.5,-0.5,0,1);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		glMainptr->reDraw();
	}
	return TRUE;
}

// idle callback
gint GLMain::cbIdle(gpointer) {

	if (!glMainptr->playing) {
		gtk_idle_remove(glMainptr->idleTag);
		glMainptr->idleRegistred = false;
	}

	glMainptr->paint();
	return TRUE;
}

//! call back to create a new Duration (effect instance, group, etc..)
gint GLMain::cbNewDuration(GtkWidget *widget, char *name) {
	
	printf("new duration: %s\n", name);
	Duration *d = glMainptr->durationFactory.create(name);
	DurationContainer *father = glMainptr->trackDlg.selectedContainer();
	father->addChild(d);
	d->setStartTime(glMainptr->getTimeMilliSec());
	glMainptr->trackDlg.addDuration(d);
	glMainptr->reDraw();
	return TRUE;
}

/* keyboard callback function */
gint cbKeyboard(GtkWidget *widget, GdkEventKey *event, gpointer data) {
	
	switch(event->keyval) {
		case 27: 
		case 'q':
			exit(0);
			break;

		case ' ': cbPlayStopButton(0, glMainptr);
			  break;
		default:
			glMainptr->reDraw();
	};

	return TRUE;
}

gint cbDemoTime(GtkWidget *widget, gpointer data) {
	gfloat new_time = glMainptr->timeAdjustment->value;

	if (!ignoreSetTime && new_time != glMainptr->getTimeSec()) {
		glMainptr->setTime( new_time );

		// set the timeStart variable, so the current time will change if
		// the demo is playing.
		glMainptr->timeStart = getMilliSec() - glMainptr->getTimeMilliSec();
	}
				
	if (!glMainptr->playing) {
		glMainptr->reDraw();
	}
	return TRUE;
}

// should return FALSE if quit action is canceled
gint cbQuit(GtkWidget *widget, gpointer data) {
	gtk_main_quit();
	return FALSE;
}


void GLMain::addDialogToMenu(GtkWidget *menu, Dlg *dlg) {
	GtkWidget *item = gtk_menu_item_new_with_label(dlg->getName());
	gtk_menu_append(GTK_MENU(menu), item);
	gtk_signal_connect(GTK_OBJECT(item), "activate",
			GTK_SIGNAL_FUNC(menuWindowShowHide), dlg);
	gtk_widget_show(item);
}

/********************************************
**    Initialization and GUI drawing       **
********************************************/
	
// open the window, and start.
int GLMain::start(int *argc, char ***argv) {

	glMainptr = this;

	if (started) return 1;
	started=1;

	// intialize GTK
	//gtk_set_locale();
	gtk_init(argc, argv);

	fileManager.addDir(PACKAGE_DATA_DIR);
	fileManager.addDir(PACKAGE_SOURCE_DIR);

	// Check if OpenGL is supported. 
	if (gdk_gl_query() == FALSE) {
		g_print("OpenGL not supported\n");
		return 0;
	}

	timeAdjustment = (GtkAdjustment *) 
		gtk_adjustment_new (0.0, 0.0, MINIMUM_DEMO_DURATION/1000.0f, 1.0, 10.0, 0.0);

	// create the windows
	drawWindow = gtk_window_new( GTK_WINDOW_TOPLEVEL);
	gtk_window_set_default_size(GTK_WINDOW(drawWindow), 640, 490);
	gtk_window_set_title(GTK_WINDOW(drawWindow), "Calodox DemoNiac");

	int attrlist[] = {
		GDK_GL_RGBA,
		GDK_GL_DOUBLEBUFFER,
		GDK_GL_NONE
	};           

	// create openGL widget
	glarea = GTK_WIDGET(gtk_gl_area_new(attrlist));

	// Events for widget must be set before X Window is created 
	gtk_widget_set_events(GTK_WIDGET(glarea),
			GDK_EXPOSURE_MASK|
			GDK_BUTTON_PRESS_MASK |
			GDK_BUTTON_RELEASE_MASK |
			GDK_BUTTON_MOTION_MASK
			);

	mc = new MouseController(glarea);

	GtkWidget *menubar;
	GtkItemFactory *menuFactory = setupMenu(drawWindow, &menubar);
	GtkWidget *drawWinVBox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(drawWindow), drawWinVBox);

	GtkWidget *menuHandle = gtk_handle_box_new();
	gtk_container_add(GTK_CONTAINER(menuHandle), menubar);

	gtk_box_pack_start(GTK_BOX(drawWinVBox), menuHandle, FALSE, FALSE, 0);
	gtk_widget_show(menubar);

	menuHandle = gtk_handle_box_new();
	gtk_container_add(GTK_CONTAINER(menuHandle), toolbar.create(drawWindow));
	gtk_widget_show(menuHandle);
	gtk_box_pack_start(GTK_BOX(drawWinVBox), menuHandle, FALSE, FALSE, 0);


	gtk_box_pack_start(GTK_BOX(drawWinVBox), glarea, TRUE, TRUE, 0);
	gtk_widget_show(drawWinVBox);
	gtk_widget_show(glarea);

	// Redraw image when exposed.  
	gtk_signal_connect(GTK_OBJECT(glarea), "expose_event",
			GTK_SIGNAL_FUNC(cbDisplay), NULL);
	// When window is resized viewport needs to be resized also. 
	gtk_signal_connect(GTK_OBJECT(glarea), "configure_event",
			GTK_SIGNAL_FUNC(cbReshape), NULL);
	// Do initialization when widget has been realized. 
	gtk_signal_connect(GTK_OBJECT(glarea), "realize",
			GTK_SIGNAL_FUNC(cbInit), NULL);
	gtk_signal_connect(GTK_OBJECT(drawWindow), "key_press_event",
			GTK_SIGNAL_FUNC(cbKeyboard), NULL);

	gtk_signal_connect(GTK_OBJECT(drawWindow), "delete_event",
			GTK_SIGNAL_FUNC(cbQuit), NULL);

	// create the track dialog with the time adjustment
	trackDlg.build();
	trackDlg.show();

	durationDlg.build();

	// create the draw buffer list dialog
	drawBufferList.build();
	drawBufferList.show();
	

	// get the effects/new submenu
	GtkWidget *subMenuNew= gtk_item_factory_get_widget(menuFactory, "/New");
	createNewMenu(GTK_MENU(subMenuNew));
	
	// get the windows submenu
	GtkWidget *subMenu = gtk_item_factory_get_widget(menuFactory, "/Windows");
	if (subMenu) {
		addDialogToMenu(subMenu, &trackDlg);
		addDialogToMenu(subMenu, &drawBufferList);
	}

	playing = false;
	selectedFX=vfx[0];


	// Show everything
	gtk_widget_show_all(drawWindow);
	// add the menu
	//  createMenu();

  	// launch the loop
	gtk_main();
	return 0;  
}

/***************************************
**          Main Drawing Method       **
***************************************/

/* let's render all the fx instances */
void GLMain::reDraw() {
	if (!idleRegistred) {
		idleTag=gtk_idle_add_priority(GTK_PRIORITY_LOW, cbIdle, glMainptr);
		idleRegistred = true;
	}
}

void GLMain::paint() {
	int start, end;
	unsigned int i;


	if (!gtk_gl_area_make_current(GTK_GL_AREA(glarea))) 
		return;

	// get frame start time
	start = getMilliSec();

	if (playing) {
		// update the demo time
		float newTime = ((float)(start - timeStart)) / 1000.0;
		if (newTime >= timeAdjustment->upper) {
			// loop
			newTime = timeAdjustment->lower;
			timeStart = getMilliSec() - (int) (newTime * 1000.0);
		}
		setTime(newTime);		
	}

	demo.actionAtTime(getTimeMilliSec());

	// clear the screen if there are no effects
	if (demo.children.size() == 0) {
		glClearColor(0,0,0,1);
		glClear(GL_COLOR_BUFFER_BIT);
	}

	// display option stuff on screen, like
	// a rectangle
	mc->draw();	

	// work's done, display it
	gtk_gl_area_swapbuffers(GTK_GL_AREA(glarea));

	end = getMilliSec();
	//sprintf(string,"%05.2f", 1000.0/(double)(end - start)); 
        //fps_box->set_text(string);

}


/*!
 * adjustDemoDuration(): look when the last first level
 * duration finishes and and set demo.duration
 */
void GLMain::adjustDemoDuration() {
	
	int duration = MINIMUM_DEMO_DURATION;

	std::vector<Duration *>::iterator it;

	for (it = demo.children.begin(); it != demo.children.end(); ++it) {
		int stop = (*it)->startTime + (*it)->duration;
		if (duration < stop)
			duration = stop;
	}
	
	if (demo.duration != duration)
		demo.setDuration(duration);

	float fd = ((float) duration)/1000.0;

	if (timeAdjustment->upper != fd) {
		timeAdjustment->upper = fd;
		gtk_signal_emit_by_name(GTK_OBJECT(timeAdjustment), "changed");
	}
}

/************************************
**             Saver                **
*************************************/

void GLMain::save(Saver *s) {
    s->saveObjectBegin("DemoNiac");

    if (filename)
    	delete filename;
    filename = new char [strlen(s->getFileName())+1];
    strcpy(filename, s->getFileName());

    s->saveFloat("demoTimeSec", demoTimeSec);

    drawBufferList.save(s);
    
    // now the instances
    demo.save(s);

    s->saveObjectEnd();
}

/*************************************
**              Loader              **
*************************************/

void GLMain::load(Loader *l) {

	if (strcmp("DemoNiac", l->nextName())==0) {
		l->nextLine();

		clearAll();

		filename = new char[strlen(l->getFileName())+1];
		strcpy(filename, l->getFileName());

		float time = l->getFloat("demoTimeSec");
		l->nextLine();

		drawBufferList.load(l);

		demo.load(l);

		l->checkType(Loader::OBJECT_END);
			
		// compute options for the new time
		demoTimeSec = time +1; // this forces setTime to recompute options
		setTime(time);
	}
}

