/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * GLMain.h: GLDemoNiac main class header
 * Leto/calodox, 12.2000
 */

#ifndef _GLMAIN_H_
#define _GLMAIN_H_

#include <gtk/gtk.h>
#include <fx.h>
#include <vector>
#include "opengl.h"
#include "saver.h"
#include "dlgopt.h"
#include "mousecontroller.h"
#include "trackdlg.h"
#include "drawbufflist.h"
#include "filemanager.h"
#include "durationcontainer.h"
#include "toolbar.h"
#include "durationfactory.h"
#include "durationdlg.h"

//#include "fxinstance.h"
class FXInstance;

using namespace std;

class GLMain {
private:
	// current selected effect
	FXGeneric *selectedFX; 	

	// did we already start ?
	int started;

	// current demo time
	float demoTimeSec; // demo time in seconds
	
	// time gtk adjustment
	GtkAdjustment *timeAdjustment;
	
	// machine time corresponding to 0 demo time
	int timeStart; 

	bool playing; 	// are we playing the demo ?
	
	// the window in witch we draw the effects
	GtkWidget *drawWindow;

	// the GL Gtk widget, in which we render the demo
	GtkWidget *glarea;

	// remember the Gtk tag for idle function
	gint idleTag;
	bool idleRegistred;

	/*
	void renameInstance(FXInstance *ptr); 
	void addFXInstance(FXInstance *fxi);
	bool removeFXInstance(FXInstance *fxi);
	*/


	// Menu
	//int menu;
	//void createMenu();
	friend void cbMenu(int i);

	// the callback needs access to private members
	friend gint cbFXChoose(GtkWidget *widget, gpointer data); 
	friend gint cbKeyboard(GtkWidget *widget, GdkEventKey *event, gpointer data);
	friend gint cbNewInstance(GtkWidget *widget, gpointer data);
	friend gint cbRemoveInstance(GtkWidget *widget, gpointer data); 
	friend gint cbRenameInstance(GtkWidget *widget, gpointer *data); 
	friend gint cbDemoTime(GtkWidget *widget, gpointer data);
	friend gint cbPlayStopButton(GtkWidget *widget, gpointer data);
	friend gint cbFXInstanceCList(GtkWidget *widget, gint row, gint col,  GdkEvent *event, gpointer data);
	friend gint cbConfigureButton(GtkWidget *widget, gpointer data);
	friend gint cbUpButton(GtkWidget *widget, gpointer data);
	friend gint cbDownButton(GtkWidget *widget, gpointer data);

	void addDialogToMenu(GtkWidget *menu, Dlg *dlg);

	static gint cbIdle(void *ptr);
	void paint();

public:

	//! constructor 
	GLMain(void);

	//! destructor: call finish on all effects.
	~GLMain();

	//! kill all instances, reset application
	void GLMain::clearAll();

	//! register a new effect 
	int registerFX(FXGeneric *fx, int fxsize);

	/*! initialize the video mode and start running 
	 * the effects.
	 */
	int start(int *argc, char ***argv);

	//! reDraw schedules a repaint of the window
	void reDraw(void);

	// Time managment
	void setTime(float seconds);  //< set demoTimeSec and compute options
	void setTime(int millisec);   // set the time in milliseconds
	int getTimeMilliSec() { return (int) (demoTimeSec * 1000.0); }
	float getTimeSec() { return demoTimeSec; }
	GtkAdjustment *getTimeAdjustment() { return timeAdjustment; }
	void adjustDemoDuration();

	// dump & load object to file
	void save(Saver *s);
	void load(Loader *l);

	//! all the FXs
	vector<FXGeneric *> vfx;
	
	//! The demo is contained in this duration
	DurationContainer demo;

	//! the mouse controller for the GL window
	MouseController *mc;

	//! the track dialog, containing keys & timelines.
	TrackDlg trackDlg;

	//! the toolbar for time control.
	Toolbar toolbar;

	//! the buffer list (is a dialog)
	DrawBufferList drawBufferList;

	//! the file manager.
	FileManager fileManager;

	//! the duration factory knows all type of
	//  duration, with their name & constructor.
	DurationFactory durationFactory;

	//! The duration properties dialog, which is
	//  generic and has to be specialized with setDuration().
	DurationDlg durationDlg;
	
	//! file to work on
	// it's public, because it's used by a lot of callbacks
	char *filename;


	static gint cbNewDuration(GtkWidget *widget, char *name);
};

//! used for callbacks to know what object to use.
extern GLMain *glMainptr;

/*
 * callbacks prototypes 
 */

gint cbDisplay(GtkWidget *widget, GdkEventExpose *event);
gint cbInit(GtkWidget *widget);
gint cbPlayStopButton(GtkWidget *widget, gpointer data);
gint cbReshape(GtkWidget *widget, GdkEventConfigure *event);
gint cbIdle(gpointer);
gint cbNewInstance(GtkWidget *widget, gpointer data);
gint cbDemoTime(GtkWidget *widget, gpointer data);
gint cbQuit(GtkWidget *widget, gpointer data);

#endif
