/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Key track dialog window for calodox DemoNiac
 * leto@calodox.org, 8.2001
 */

#include <assert.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include "gtktimeline.h"
#include "glmain.h"
#include "trackdlg.h"
#include "genericoption.h"
#include "interpolator.h"
#include "report.h"
#include "menu.h"

#define EVENT_METHOD(i, x) GTK_WIDGET_CLASS(GTK_OBJECT(i)->klass)->x

static GtkAdjustment *currentKey;

//! this lock prevent the start & stop keys callback
//  to modify the duration or any GtkAdjustment.
static bool lockStartStopKeys=false; 

//! This lock prevent setNewTime() to operate.
static bool lockSetNewTime = false;

TrackDlg::TrackDlg() : pixSize(500) {
}

TrackDlg::~TrackDlg() {
}


void TrackDlg::build() {

	// create the window
	window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_window_set_default_size(GTK_WINDOW(window), 640, 160);
	gtk_window_set_title(GTK_WINDOW(window), "Demoniac: key editor");
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			GTK_SIGNAL_FUNC(Dlg::cbDeleteWindow), (Dlg *) this);
	// make sure to get motion_notify events
	gtk_widget_set_events( window, GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK );

	// create a vertical box 
	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), vbox);


	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);

	// create the zoom in / zoom out buttons
	GtkWidget *zoomIn = gtk_button_new_with_label("Zoom In");
	gtk_signal_connect(GTK_OBJECT(zoomIn), "clicked",
			GTK_SIGNAL_FUNC(cbZoomIn), this);
	gtk_box_pack_start(GTK_BOX(hbox), zoomIn, TRUE, TRUE, 2);
	GtkWidget *zoomOut = gtk_button_new_with_label("Zoom Out");
	gtk_signal_connect(GTK_OBJECT(zoomOut), "clicked",
			GTK_SIGNAL_FUNC(cbZoomOut), this);
	gtk_box_pack_start(GTK_BOX(hbox), zoomOut, TRUE, TRUE, 2);

	// get time adjustment
	GtkAdjustment *adj = glMainptr->getTimeAdjustment();
	assert(adj != 0);

	// create an horizontal ruler
	ruler = gtk_hruler_new();
	gtk_ruler_set_range( GTK_RULER(ruler), 0, adj->upper, 0, adj->upper);
	gtk_signal_connect(GTK_OBJECT(adj), "changed", 
			GTK_SIGNAL_FUNC(cbTimeChanged), this);

	gtk_widget_set_usize( ruler, pixSize, -2);
	gtk_box_pack_end(GTK_BOX(hbox), ruler, FALSE, FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);


	// create the scrolled window
	GtkWidget *scrWin = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrWin),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(vbox), scrWin, TRUE, TRUE, 5);

	// create the tree widget
	root = GTK_TREE(gtk_tree_new());

	gtk_tree_set_selection_mode(GTK_TREE(root), GTK_SELECTION_SINGLE);

	//gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(root), TRUE, TRUE, 0);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrWin), GTK_WIDGET(root));

	// create the scrollbar for time range control.
	// by default, zoom out to display everything.
	timeRange = GTK_ADJUSTMENT(gtk_adjustment_new(adj->value, 0, adj->upper, 
			adj->step_increment, adj->page_increment, adj->upper));
	gtk_signal_connect(GTK_OBJECT(timeRange), "changed",
			GTK_SIGNAL_FUNC(cbZoom), this);
	gtk_signal_connect(GTK_OBJECT(timeRange), "value_changed",
			GTK_SIGNAL_FUNC(cbZoom), this);
	// create the horizontal scroll bar for time
	GtkWidget *scroll = gtk_hscrollbar_new(GTK_ADJUSTMENT(timeRange));
	gtk_widget_set_usize( scroll, pixSize, -2);
	hbox = gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(hbox), scroll, FALSE, FALSE, 0);

	gtk_widget_show_all(vbox);
	
	// now create the popup menus
	contextMenuTrack = gtk_menu_new();
	
	GtkWidget *item = gtk_menu_item_new_with_label("Delete");
	gtk_menu_append(GTK_MENU(contextMenuTrack), item);
	gtk_signal_connect(GTK_OBJECT(item), "activate",
			GTK_SIGNAL_FUNC(cbRemoveKey), this);
	gtk_widget_show_all(contextMenuTrack);

	// create Duration context menu
	contextMenuDuration = gtk_menu_new();
	item = gtk_menu_item_new_with_label("New");
        gtk_menu_append(GTK_MENU(contextMenuDuration), item);
	GtkWidget *subMenu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), subMenu);
	createNewMenu(GTK_MENU(subMenu));
	item = gtk_menu_item_new_with_label("Properties");
	gtk_menu_append(GTK_MENU(contextMenuDuration), item);
	gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(cbDurationContextMenu), (void *) "Properties");
	item = gtk_menu_item_new_with_label("Remove");
	gtk_menu_append(GTK_MENU(contextMenuDuration), item);
	gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(cbDurationContextMenu), (void *) "Remove");

	gtk_widget_show_all(contextMenuDuration);

	// create option context menu
	contextMenuOption = gtk_menu_new();
	radioInterp[0] = gtk_radio_menu_item_new_with_label(0, "Static");
	gtk_menu_append(GTK_MENU(contextMenuOption), radioInterp[0]);
	gtk_signal_connect(GTK_OBJECT(radioInterp[0]), "activate", GTK_SIGNAL_FUNC(cbChooseInterpolator), &fixedInterpolator);
	GSList *group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(radioInterp[0]));

	radioInterp[1] = gtk_radio_menu_item_new_with_label(group, "Last Key");
	gtk_signal_connect(GTK_OBJECT(radioInterp[1]), "activate", GTK_SIGNAL_FUNC(cbChooseInterpolator), &lastValueInterpolator);
	gtk_menu_append(GTK_MENU(contextMenuOption), radioInterp[1]);
	group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(radioInterp[1]));

	radioInterp[2] = gtk_radio_menu_item_new_with_label(group, "Linear");
	gtk_signal_connect(GTK_OBJECT(radioInterp[2]), "activate", GTK_SIGNAL_FUNC(cbChooseInterpolator), &linearInterpolator);
	gtk_menu_append(GTK_MENU(contextMenuOption), radioInterp[2]);
	group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(radioInterp[2]));

	radioInterp[3] = gtk_radio_menu_item_new_with_label(group, "Smooth");
	gtk_signal_connect(GTK_OBJECT(radioInterp[3]), "activate", GTK_SIGNAL_FUNC(cbChooseInterpolator), &smoothInterpolator);
	gtk_menu_append(GTK_MENU(contextMenuOption), radioInterp[3]);

	item = gtk_menu_item_new_with_label("Properties");
	gtk_menu_append(GTK_MENU(contextMenuOption), item);
	gtk_signal_connect(GTK_OBJECT(item), "activate",
			GTK_SIGNAL_FUNC(cbOptionProperties), this);
	gtk_widget_show_all(contextMenuOption);
}

const char *TrackDlg::getName() {
	return "Key editor";
}

void TrackDlg::addDuration(Duration *newDuration) {

	DurationContainer *father;

	if (newDuration->father == 0)
		father = &glMainptr->demo;
	else
		father = newDuration->father;

	// create the node if needed
	if (!newDuration->guiData) {
		
		// create a new item
		newDuration->guiData = gtk_tree_item_new();
		gtk_widget_show(GTK_WIDGET(newDuration->guiData));
		gtk_object_set_data(GTK_OBJECT(newDuration->guiData), "DurationPtr", newDuration);

		gtk_signal_connect(GTK_OBJECT(newDuration->guiData), "button-press-event",
				GTK_SIGNAL_FUNC(cbDurationClicked), newDuration);
		

		// create the adjustment which contains the current time
		// relative to this duration
		if (newDuration->father == 0) {
			gtk_object_set_data(GTK_OBJECT(newDuration->guiData), "DurationAdjustment", 
					glMainptr->getTimeAdjustment());
		} else {
			GtkAdjustment *adj = (GtkAdjustment *) 
				gtk_adjustment_new((float) newDuration->startTime/1000.0, 
						-FLT_MAX, FLT_MAX, 1, 2, 3);
			gtk_object_set_data(GTK_OBJECT(newDuration->guiData), "DurationAdjustment", adj);

			// make this duration sensitive to time changing.
			gtk_signal_connect(GTK_OBJECT(glMainptr->getTimeAdjustment()), 
					"value_changed",
					GTK_SIGNAL_FUNC(cbTimeChangedForDuration), newDuration);
			
			// make this duration change time when needed.
			gtk_signal_connect(GTK_OBJECT(adj), 
					"value_changed",
					GTK_SIGNAL_FUNC(cbSetNewTime), newDuration);

			GtkWidget *fatherAdj=0;

			if (newDuration->father && newDuration->father->guiData) {
				fatherAdj = (GtkWidget *) gtk_object_get_data(
						GTK_OBJECT(newDuration->father->guiData), 
						"DurationAdjustment");
			}
			if (fatherAdj == 0)
				fatherAdj = (GtkWidget *) glMainptr->getTimeAdjustment();

			assert(GTK_IS_ADJUSTMENT(fatherAdj));

			//gtk_signal_connect(GTK_OBJECT(fatherAdj), "value_changed",
			//		GTK_SIGNAL_FUNC(cbAbsoluteAdjustment), newDuration);
		}

		buildDurationTreeItem(newDuration);

		GtkWidget *r;
		if (father->guiData) {
			r= GTK_TREE_ITEM(father->guiData)->subtree;

			if (r == 0) {
				// create a subtree
				r = gtk_tree_new();
				gtk_tree_item_set_subtree(GTK_TREE_ITEM(father->guiData), r);
			}

		} else {
			r = GTK_WIDGET(root);
		}

		gtk_tree_append(GTK_TREE(r), GTK_WIDGET(newDuration->guiData));

		int stop = newDuration->startTime + newDuration->duration;
		stop = newDuration->father->relToAbs(stop);
		makeInViewRange( ((float)stop)/1000.0 );

		// if the duration is not a container, it's an FXInstance.
		// we have to add all options.
		if (!newDuration->isContainer()) {
			addOptions((FXInstance *) newDuration);
		}
	}

	gtk_widget_show(GTK_WIDGET(newDuration->guiData));
}

void TrackDlg::buildDurationTreeItem(Duration *duration) {
	
	GtkTreeItem *item = GTK_TREE_ITEM(duration->guiData);
	GtkAdjustment *adj;
	
	if (duration->father->father == 0)
		adj = glMainptr->getTimeAdjustment();
	else {
		adj = GTK_ADJUSTMENT(
			gtk_object_get_data(GTK_OBJECT(duration->father->guiData), "DurationAdjustment"));
	}

	// the hbox containing the label and the timeline
	GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(item), hbox);

	// the label
	GtkWidget *label = gtk_label_new(duration->name);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);

	// the timeline 
	GtkWidget *timeline = gtk_time_line_new(adj);
	gtk_widget_set_usize(timeline, pixSize, -2);
	gtk_box_pack_end(GTK_BOX(hbox), timeline, FALSE, FALSE, 0);

	gtk_signal_connect_object( GTK_OBJECT(timeline), "motion_notify_event",
			(GtkSignalFunc)EVENT_METHOD(ruler, motion_notify_event),
			GTK_OBJECT(ruler) );

	// add the start & stop keys
	float start = (float)duration->startTime/1000.0;
	GtkAdjustment *startAdj = GTK_ADJUSTMENT(gtk_adjustment_new(
				start , 0, FLT_MAX, 1, 2, 3));
	GtkAdjustment *stopAdj = GTK_ADJUSTMENT(gtk_adjustment_new(
				start + ((float)duration->duration)/1000.0, 0, FLT_MAX, 1, 2, 3));

	gtk_signal_connect(GTK_OBJECT(timeRange), "changed",
			GTK_SIGNAL_FUNC(cbZoomTimeLineDuration), duration);
	gtk_signal_connect(GTK_OBJECT(timeRange), "value_changed",
			GTK_SIGNAL_FUNC(cbZoomTimeLineDuration), duration);

	gtk_object_set_data(GTK_OBJECT(duration->guiData), "DurationStartAdjustment", 
			startAdj);

	gtk_object_set_data(GTK_OBJECT(duration->guiData), "DurationStopAdjustment", 
			stopAdj);

	// watch the user move the keys
	gtk_signal_connect(GTK_OBJECT(startAdj), "value_changed", 
			GTK_SIGNAL_FUNC(cbStartKeyMoved), duration);
	gtk_signal_connect(GTK_OBJECT(stopAdj), "value_changed", 
			GTK_SIGNAL_FUNC(cbStopKeyMoved), duration);

	gtk_time_line_add_key(GTK_TIME_LINE(timeline), startAdj);
	gtk_time_line_add_key(GTK_TIME_LINE(timeline), stopAdj);

	cbZoomTimeLineDuration(timeRange, duration);

	gtk_widget_show_all(hbox);
}

void TrackDlg::updateDuration(Duration *duration) {

	assert(duration != 0);

	if (duration->guiData) {
		// update start & stop keys
		
		// Lock the callbacks, to prevent them from changing anything.
		lockStartStopKeys = true;
		void *startAdj = gtk_object_get_data(GTK_OBJECT(duration->guiData),
				"DurationStartAdjustment");
		float start = ((float)duration->startTime)/1000.0;
		gtk_adjustment_set_value(GTK_ADJUSTMENT(startAdj), start);
		
		void *stopAdj = gtk_object_get_data(GTK_OBJECT(duration->guiData),
				"DurationStopAdjustment");
		float stop = start + ((float)duration->duration)/1000.0;
		gtk_adjustment_set_value(GTK_ADJUSTMENT(stopAdj), stop);
		
		lockStartStopKeys = false;

		// compute the absolute stop time
		assert(duration->father != 0);
		int astop = duration->father->relToAbs( (int) (stop *1000.0) );

		makeInViewRange( ((float)astop)/1000.0);

		if (duration->father->father == 0 && astop > glMainptr->demo.duration)
			glMainptr->adjustDemoDuration();

		// re-range the duration timeline
		cbZoomTimeLineDuration(timeRange, duration);
	}

	if (duration->isContainer()) {
		DurationContainer *dc = (DurationContainer *) duration;

		std::vector<Duration *>::iterator it;

		for (it = dc->children.begin(); it != dc->children.end(); ++it) {
			updateDuration(*it);
		}
	} else {
		FXInstance *fxi = (FXInstance *) duration;	
		std::vector<GenericOption *>::iterator it;

		for (it = fxi->opts.begin(); it != fxi->opts.end(); ++it) {
			if ((*it)->track && (*it)->track->timeline)
				cbZoomTimeLine(timeRange, *it);
		}
	}	
}

void TrackDlg::renameDuration(Duration *duration) {

	assert(duration!= 0);
	assert(duration->guiData !=0);

	GtkWidget *treeItem = (GtkWidget *) duration->guiData;
	GList *l = gtk_container_children(GTK_CONTAINER(treeItem));
	GtkWidget *w = GTK_WIDGET(l->data);

	if (GTK_IS_LABEL(w)) {
		gtk_label_set_text(GTK_LABEL(w), duration->name);
		return;
	}

	if (GTK_IS_BOX(w)) {
		l = gtk_container_children(GTK_CONTAINER(w));
		GtkWidget *label = (GtkWidget *) l->data;

		if (GTK_IS_LABEL(label))
			gtk_label_set_text(GTK_LABEL(label), duration->name);

		return;
	}

	fatal("unable to rename duration");
}

void TrackDlg::removeFXInstance(FXInstance *fxi) {
	std::vector<GenericOption *>::iterator it;

	/*
	for (it = fxi->opts.begin(); it != fxi->opts.end(); it++)
		removeOption(*it);

	gtk_tree_remove_item(root, fxi->treeItem);
	fxi->treeItem=0;
	*/
}

void TrackDlg::addOptions(FXInstance *fxi) {
	std::vector<GenericOption *>::iterator it;

	for (it = fxi->opts.begin(); it != fxi->opts.end(); it++) {
		addOption(*it);
	}
}

//void TrackDlg::addAnimatedOption(GenericOption *opt) {
void TrackDlg::addOption(GenericOption *opt) {

	Track *track = opt->track;

	if (track == 0)
		return;

	if (opt->treeItem) {
		gtk_tree_remove_item(GTK_TREE(GTK_TREE_ITEM(opt->owner->guiData)->subtree), GTK_WIDGET(opt->treeItem));
		opt->treeItem = 0;
	}
	
	opt->treeItem = gtk_tree_item_new();
	gtk_signal_connect(GTK_OBJECT(opt->treeItem), "button-press-event",
			GTK_SIGNAL_FUNC(cbOptionClicked), opt);

	GtkWidget *tree= GTK_TREE_ITEM(opt->owner->guiData)->subtree;
	if (tree == 0) {
		tree = gtk_tree_new();
		gtk_tree_item_set_subtree(GTK_TREE_ITEM(opt->owner->guiData), tree);
	}

	// create the hbox
	GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(opt->treeItem), hbox);

	// the label
	GtkWidget *label = gtk_label_new(opt->name);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	
	/*
	GtkRequisition req;
	gtk_widget_size_request(label, &req);
	printf("label requisition: %d\n", req.width);
	*/

	// the timeline

	if (track->timeline == 0) {
		// get time adjustment
		GtkAdjustment *adj = (GtkAdjustment *) gtk_object_get_data(
				GTK_OBJECT(opt->owner->guiData), "DurationAdjustment");
		assert(GTK_IS_ADJUSTMENT(adj));

		track->timeline = gtk_time_line_new(adj);
		gtk_widget_set_usize(track->timeline, pixSize, -2);
		gtk_box_pack_end(GTK_BOX(hbox), track->timeline, FALSE, FALSE, 0);

		gtk_signal_connect_object( GTK_OBJECT(track->timeline), "motion_notify_event",
				(GtkSignalFunc)EVENT_METHOD(ruler, motion_notify_event),
				GTK_OBJECT(ruler) );
		
		gtk_signal_connect(GTK_OBJECT(track->timeline), "key_press",
				GTK_SIGNAL_FUNC(cbKeyPress), track);

		gtk_signal_connect(GTK_OBJECT(timeRange), "changed",
				GTK_SIGNAL_FUNC(cbZoomTimeLine), opt);
		
		gtk_signal_connect(GTK_OBJECT(timeRange), "value_changed",
				GTK_SIGNAL_FUNC(cbZoomTimeLine), opt);
		cbZoomTimeLine(timeRange, opt);

		// now add the new treeItem to the effect branch
		gtk_tree_append(GTK_TREE(tree), opt->treeItem);

		// add all the keys 
		for (Key *k = opt->track->first; k != 0; k=k->next) {
			addKey(opt->track, k);
		}
	}
	gtk_widget_show_all(GTK_WIDGET(opt->treeItem));
	if (track->interpolator->getType() == 1)
		gtk_widget_hide(GTK_WIDGET(track->timeline));

}

void TrackDlg::updateOption(GenericOption *opt) {

	if (opt->track && opt->track->interpolator->getType() != 1) 
		gtk_widget_show(GTK_WIDGET(opt->track->timeline));
	else
		gtk_widget_hide(GTK_WIDGET(opt->track->timeline));
}

void TrackDlg::removeDuration(Duration *d) {

	assert(d!=0);

	assert(d->father!=0 /* can't remove root duration !*/);

	if (d->isContainer()) {
		// d is a container. Remove all it's children first.
		std::vector<Duration *>::iterator it;
		DurationContainer *dc = (DurationContainer *) d;

		for (it = dc->children.begin(); it!= dc->children.end(); ++it) 
			removeDuration(*it);

	} else {
		// it's a FXInstance.
		FXInstance *fxi = (FXInstance *) d;
		std::vector<GenericOption *>::iterator it;

		for (it = fxi->opts.begin(); it!=fxi->opts.end(); ++it) 
			removeOption(*it);
	}

	// disconnect signals
	gtk_signal_disconnect_by_data(
			GTK_OBJECT(glMainptr->getTimeAdjustment()), d);

	gtk_signal_disconnect_by_data (GTK_OBJECT (timeRange), d);

	// now remove the GtkTreeItem of this duration
	GtkTree *tree;
	if (d->father->father == 0)
		tree = root;
	else
		tree = GTK_TREE(GTK_TREE_ITEM(d->father->guiData)->subtree);

	gtk_tree_remove_item(tree, GTK_WIDGET(d->guiData));
	d->guiData=0;
}

void TrackDlg::removeOption(GenericOption *opt) {
	if (opt == 0 || opt->track == 0 || opt->treeItem == 0)
		return;


	for (Key *k = opt->track->first; k != 0; k=k->next) {
		removeKey(opt->track, k);
	}
		
	if (opt->track->timeline)
		gtk_signal_disconnect_by_data(GTK_OBJECT(timeRange), opt->track->timeline);

	//gtk_tree_remove_items(GTK_TREE(tree), &l);
	opt->treeItem = 0;
	opt->track->timeline = 0;
}

void TrackDlg::addKey(Track *track, Key *key) {

	if (track->timeline == 0)
		return;

	assert (key->adj == 0); 

	debug("added key");

	key->adj = GTK_ADJUSTMENT(gtk_adjustment_new(((float)key->time)/1000.0, 0, FLT_MAX, 1, 2, 3));
	gtk_signal_connect(GTK_OBJECT(key->adj), "value_changed", 
			GTK_SIGNAL_FUNC(cbKeyMoved), key);

	gtk_time_line_add_key(GTK_TIME_LINE(track->timeline), key->adj);
	glMainptr->trackDlg.makeInViewRange(key->adj->value);

}

void TrackDlg::removeKey(Track *track, Key *key) {
	if (track->timeline == 0 )
		return;

	assert(key->adj != 0);

	debug("removed key");

	gtk_time_line_remove_key(GTK_TIME_LINE(track->timeline), key->adj);

	gtk_object_unref(GTK_OBJECT(key->adj));

	key->adj = 0;
}

void TrackDlg::moveKey(Key *key) {
	
	if (key->adj == 0)
		return;

	if ((int) (key->adj->value * 1000.0) != key->time) {

		// update the adjustment, and emit signal
		key->adj->value = ((float) key->time) / 1000.0;

		gtk_signal_emit_by_name(GTK_OBJECT(key->adj), "value_changed");
	}
}

void TrackDlg::cbKeyMoved(GtkAdjustment *adj, gpointer data) {
	debug("key moved");

	Key *k = (Key *) data;

	assert(k->owner);

	gtk_signal_handler_block_by_data(GTK_OBJECT(adj), data);

	k->owner->moveKey(k, (int) (adj->value * 1000.0));

	gtk_signal_handler_unblock_by_data(GTK_OBJECT(adj), data);

}

void TrackDlg::cbKeyPress(GtkTimeLine *timeline, GdkEventButton *event, GtkAdjustment *adj, gpointer data) {
	
	currentKey = adj;
	glMainptr->trackDlg.context = data;
	gtk_menu_popup(GTK_MENU(glMainptr->trackDlg.contextMenuTrack), 0, 0, 0, 0, 3, event->time);
}

void TrackDlg::cbRemoveKey(GtkTimeLine *timeline, gpointer data) {
	Track *track = (Track *) ((TrackDlg *) data)->context;

	track->removeKey((int) (currentKey->value * 1000.0));
	
	debug("cbRemoveKey");
	
}

void TrackDlg::cbTimeChanged(GtkAdjustment *adj, TrackDlg *obj) {
	if (adj->upper > obj->timeRange->upper) {
		obj->timeRange->upper = adj->upper;
		gtk_signal_emit_by_name(GTK_OBJECT(obj->timeRange), "changed");
	}
}

//! make the adjustment representing current time relative to this
//  duration follow the global time adjustment.
void TrackDlg::cbTimeChangedForDuration(GtkAdjustment *adj, Duration *d) {

	assert(d !=0 && d->father !=0 && d->guiData != 0);
	
	float time = ((float) d->absToRel((int)(adj->value*1000.0)))/1000.0;

	GtkAdjustment *dAdj = GTK_ADJUSTMENT(gtk_object_get_data(
				GTK_OBJECT(d->guiData), "DurationAdjustment"));
	assert(dAdj != 0);

	lockSetNewTime = true;
	gtk_adjustment_set_value(dAdj, time);
	lockSetNewTime = false;
}

void TrackDlg::cbTimeChangedForOption(GtkAdjustment *adj, GenericOption *opt) {

	assert(opt !=0 && opt->owner !=0);

	if (opt->track && opt->track->timeline) {
		float time = ((float) opt->owner->absToRel((int)(adj->value*1000.0)))/1000.0;

		GtkAdjustment *optAdj = gtk_time_line_get_adjustment( 
				GTK_TIME_LINE(opt->track->timeline));

		assert(optAdj != 0);

		gtk_adjustment_set_value(optAdj, time);
	}
}

//! this callback will detect when a user click on a time line to change
//  current time, and call glMainptr->setTime().
void TrackDlg::cbSetNewTime(GtkAdjustment *relAdj, Duration *d) {

	int relTime = (int)(relAdj->value *1000.0);

	if (!lockSetNewTime && relTime != glMainptr->getTimeMilliSec()) {
		lockSetNewTime = true;
		glMainptr->setTime(d->relToAbs(relTime));
		lockSetNewTime = false;
	}
}

void TrackDlg::cbZoom(GtkAdjustment *adj, TrackDlg *obj) {

	gtk_ruler_set_range( GTK_RULER(obj->ruler),
			adj->value, 
			adj->value + adj->page_size,
			adj->value, adj->upper);
}

void TrackDlg::cbZoomTimeLine(GtkAdjustment *adj, GenericOption *opt) {
	GtkTimeLine *timeLine = GTK_TIME_LINE( opt->track->timeline );

	// make it relative to owner's startTime.
	int rel = opt->owner->absToRel((int)(adj->value * 1000.0));
	gfloat val = ((gfloat)rel)/1000.0;

	gtk_time_line_set_range(timeLine, val, val + adj->page_size);

#ifdef DEBUG_TRACKDLG
	printf("Ranging option %s: %f - %f\n", opt->name, val, val+adj->page_size);
#endif 
}

//! all durations have timelines for their start & stop keys that are sensitive 
// to a global adjustment describing the time window shown.
void TrackDlg::cbZoomTimeLineDuration(GtkAdjustment *adj, Duration *duration) {
	assert(duration != 0 && duration->father != 0);

	// the adjustment is global time. Make it relative
	// to the duration father. 
	int rel = duration->father->absToRel((int)(adj->value * 1000.0));
	float val = ((float)rel)/1000.0;

	// find the timeline corresponding to this duration, and update it.
	GList *l = gtk_container_children(GTK_CONTAINER(duration->guiData)); // look in the GtkTreeItem
	l = gtk_container_children(GTK_CONTAINER(l->data)); // look in the GtkHBox
	l = l->next;	// skip the label
	gtk_time_line_set_range(GTK_TIME_LINE(l->data), val, val + adj->page_size);
#ifdef DEBUG_TRACKDLG
	printf("Ranging duration %s: %f - %f\n", duration->name, val, val+adj->page_size);
#endif
}

void TrackDlg::cbZoomIn(GtkWidget *widget, TrackDlg *obj) {

	obj->timeRange->page_size /= 2.0f;
	gtk_signal_emit_by_name(GTK_OBJECT(obj->timeRange), "changed");
}

void TrackDlg::cbZoomOut(GtkWidget *widget, TrackDlg *obj) {

	obj->timeRange->page_size = obj->timeRange->page_size * 2.0f > obj->timeRange->upper ?
		obj->timeRange->upper : obj->timeRange->page_size * 2.0f;
	gtk_signal_emit_by_name(GTK_OBJECT(obj->timeRange), "changed");
}
			
gint TrackDlg::cbDurationClicked(GtkWidget *widget, GdkEventButton *event, Duration *duration) {

	if (event->button == 3) {
		gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button-press-event");
		glMainptr->trackDlg.context = duration;
		
		// manually select the duration
		GtkTree *root = GTK_TREE(glMainptr->trackDlg.root);
		GList *l = root->selection;
		if (l) {
			gtk_tree_unselect_child(root, GTK_WIDGET(l->data));
		}
		gtk_tree_select_child(root, widget);

		gtk_menu_popup(GTK_MENU(glMainptr->trackDlg.contextMenuDuration), 0, 0, 0, 0, 3, event->time);
		return TRUE;
	}
	return FALSE;
}

gint TrackDlg::cbOptionClicked(GtkWidget *widget, GdkEventButton *event, void *data) {

	if (event->button == 3) {
		gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button-press-event");
		glMainptr->trackDlg.context = data;
		GenericOption *opt = (GenericOption *) data;
		
		// disable linear & smooth if we don't know how
		// to interpolate...
		if (opt->track && opt->track->canInterpolate()) {
			gtk_widget_show(GTK_WIDGET(glMainptr->trackDlg.radioInterp[2]));
			gtk_widget_show(GTK_WIDGET(glMainptr->trackDlg.radioInterp[3]));
		} else {
			gtk_widget_hide(GTK_WIDGET(glMainptr->trackDlg.radioInterp[2]));
			gtk_widget_hide(GTK_WIDGET(glMainptr->trackDlg.radioInterp[3]));
		}

		// activate the right interpolator type
		for (int i=0; i<4;++i) {
			if (opt->track && opt->track->interpolator) {
				gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
					glMainptr->trackDlg.radioInterp[i]),
					opt->track->interpolator->getType()-1 == i ? 1 : 0);
			}
		}

		gtk_menu_popup(GTK_MENU(glMainptr->trackDlg.contextMenuOption), 0, 0, 0, 0, 3, event->time);
		return TRUE;
	}
	return FALSE;
}

void TrackDlg::cbDurationContextMenu(GtkWidget *widget, char *data) {

	Duration *d = (Duration *) glMainptr->trackDlg.context;

	assert(d!= 0);

#ifdef DEBUG_TRACKDLG
	printf("Context Menu: Duration %s, %s\n", d->name, data);
#endif

	if (strcmp(data, "Properties")==0) {
		glMainptr->durationDlg.setDuration(d);
		glMainptr->durationDlg.show();
		return;
	}
	if (strcmp(data, "Remove")==0) {
		glMainptr->trackDlg.removeDuration(d);
		d->father->removeChild(d);
		glMainptr->reDraw();
		return;
	}
	fatal("unknown duration context menu callback !");
}

void TrackDlg::cbOptionProperties(GtkWidget *widget, void *data) {

	GenericOption *opt = (GenericOption *) glMainptr->trackDlg.context;

#ifdef DEBUG_TRACKDLG
	printf("Properties of option %s\n", opt->name);
#endif

	opt->owner->dlg->show();
}

Duration *TrackDlg::selectedDuration() {
	GList *l;

	l = root->selection;

	if (l==0) return &glMainptr->demo;

	GtkWidget *item = (GtkWidget *) l->data;

	return (Duration *) gtk_object_get_data(GTK_OBJECT(item), "DurationPtr");
}

DurationContainer *TrackDlg::selectedContainer() {
	GList *l;

	l = root->selection;

	if (l==0) return &glMainptr->demo;

	GtkWidget *item = (GtkWidget *) l->data;

	Duration *d = (Duration *) gtk_object_get_data(GTK_OBJECT(item), "DurationPtr");

	// d is not a container. 
	if (d==0)
		return &glMainptr->demo;

	if (!d->isContainer()) 
		return (DurationContainer *) d->father;

	return (DurationContainer *) d;
}

void TrackDlg::cbStartKeyMoved(GtkAdjustment *adj, Duration *duration) {

	int t = (int)(adj->value * 1000.0);

	if (lockStartStopKeys || t == duration->startTime) return;

	duration->setStartTime(t);

	GtkAdjustment *stopAdj = GTK_ADJUSTMENT( gtk_object_get_data(
				GTK_OBJECT(duration->guiData), 
				"DurationStopAdjustment"));

	float stop = adj->value + ((float) duration->duration) / 1000.0;

#ifdef DEBUG_TRACKDLG
	printf("setting start time of %s to %f (%d). Duration %d. New stop: %f\n",
			duration->name,
			adj->value,
			t,
			duration->duration,
			stop);
#endif

	gtk_adjustment_set_value(stopAdj, stop);

	glMainptr->trackDlg.updateDuration(duration);
	glMainptr->durationDlg.updateGui();
	glMainptr->reDraw();
}

void TrackDlg::cbStopKeyMoved(GtkAdjustment *adj, Duration *duration) {

	int t = (int)(adj->value * 1000.0) - duration->startTime;

	if (!lockStartStopKeys && t != duration->duration) {
#ifdef DEBUG_TRACKDLG
		printf("Stopping at %f, duration of %s has been changed from %d to %d\n",
				adj->value, duration->name, duration->duration, t);
#endif
		duration->setDuration(t);

		float astop = (float) duration->father->relToAbs( (int) (adj->value *1000.0) )
			/1000.0;

		glMainptr->trackDlg.makeInViewRange(astop);
		glMainptr->durationDlg.updateGui();
		glMainptr->reDraw();
	}
}

void TrackDlg::makeInViewRange(float absTime) {
	// check if the duration is outside the view range
	if (absTime > timeRange->upper) {
		timeRange->upper = absTime;
		gtk_signal_emit_by_name(GTK_OBJECT(timeRange), "changed");
	} 
}

void TrackDlg::cbChooseInterpolator(GtkWidget *widget, Interpolator * interp) {
	GenericOption *opt = (GenericOption *) glMainptr->trackDlg.context;

	assert(opt->track !=0);
	opt->track->setInterpolator(interp);
	glMainptr->trackDlg.updateOption(opt);
}
