/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Key track dialog window for calodox DemoNiac
 * leto@calodox.org, 8.2001
 */

#ifndef _TRACKDLG_H_
#define _TRACKDLG_H_

#include "duration.h"
#include "fxinstance.h"
#include "durationcontainer.h"
#include "genericoption.h"
#include "dlg.h"
#include "track.h"
#include "interpolator.h"

/*!
 * this class do the work of creating and managing
 * the key track dialog, instead of GLMain.
 */
class TrackDlg : public Dlg{
private:
	GtkTree *root;
	GtkWidget *ruler;
	GtkAdjustment *timeRange;
	int pixSize;

	//! pointer to data describing the context,
	//  for context menu callbacks.
	void *context;
	GtkWidget *contextMenuDuration;
	GtkWidget *contextMenuTrack;
	GtkWidget *contextMenuOption;

	GtkWidget *radioInterp[4];

	static void cbKeyPress(GtkTimeLine *timeline, GdkEventButton *event, GtkAdjustment *adj, gpointer data);
	static void cbKeyMoved(GtkAdjustment *adj, gpointer data);
	static void cbRemoveKey(GtkTimeLine *timeline, gpointer data);
	static void cbTimeChanged(GtkAdjustment *adj, TrackDlg *obj);
	/*
	static void cbAbsoluteAdjustment(GtkAdjustment *fatherAdj, Duration *d);
	static void cbDurationAdjustmentChanged(GtkAdjustment *rel, Duration *duration);
	*/

	static void cbTimeChangedForDuration(GtkAdjustment *adj, Duration *d);
	static void cbTimeChangedForOption(GtkAdjustment *adj, GenericOption *opt);
	static void cbSetNewTime(GtkAdjustment *relAdj, Duration *d);
	
	static void cbZoom(GtkAdjustment *adj, TrackDlg *obj);
	static void cbZoomTimeLine(GtkAdjustment *adj, GenericOption *opt);
	static void cbZoomTimeLineDuration(GtkAdjustment *adj, Duration *duration);
	static void cbZoomIn(GtkWidget *widget, TrackDlg *obj);
	static void cbZoomOut(GtkWidget *widget, TrackDlg *obj);
	static gboolean cbDurationClicked(GtkWidget *widget, GdkEventButton *event, Duration *duration);
	static gboolean cbOptionClicked(GtkWidget *widget, GdkEventButton *event, void *data);
	static void cbDurationContextMenu(GtkWidget *widget, char *data);
	static void cbOptionProperties(GtkWidget *widget, void *data);
	static void cbChooseInterpolator(GtkWidget *widget, Interpolator * interp);

	void addOption(GenericOption *opt);
	void removeOption(GenericOption *opt);

	void buildDurationTreeItem(Duration *duration);
	static void cbStartKeyMoved(GtkAdjustment *adj, Duration *duration);
	static void cbStopKeyMoved(GtkAdjustment *adj, Duration *duration);

public:
	TrackDlg();
	~TrackDlg();
	
	virtual void build();
	virtual const char *getName();

	void addDuration(Duration *newDuration);
	void updateDuration(Duration *duration);
	void renameDuration(Duration *duration);
	void removeDuration(Duration *duration);
	void addOptions(FXInstance *fxi);
	void removeFXInstance(FXInstance *fxi);

	void updateOption(GenericOption *opt);

	static void addKey(Track *track, Key *key);
	static void removeKey(Track *track, Key *key);
	static void moveKey(Key *key);
	void makeInViewRange(float absTime);

	Duration *selectedDuration();
	DurationContainer *selectedContainer();

};

#endif // _TRACKDLG_H_
